// Olika sett att kopiera innehållet från en vector till en annan
#include "catch.hpp"
#include <vector>
using namespace std;

// Funktion för att undersöka om innehållet är samma
bool same_content(vector<int> const & v1, vector<int> const & v2)
{
    if ( v1.size() != v2.size() )
        return false;
    // algoritmlösning
    return std::equal(begin(v1), end(v1), begin(v2));
    // egen loop
    /*  for ( vector<int>::size_type  idx{}; idx < v1.size(); ++idx )
    {
        if ( v1[idx] != v2[idx] )
           return false; 
    }
    return true;
*/
}

TEST_CASE("copy")
{
    vector<int> v {1,2,3,4,5};
    SECTION("Kopieringskonstruktor")
    {
        // bör föredras om vi har samma typ och ska skapa en ny kopia
        vector<int> v2{v};
        CHECK(same_content(v,v2));
    }

    /* två versioner som använder copy. copy ser ut ungefär såhär:
      template <typename It1, typename It2>
      void copy(It1 start, It1 stop, It2 dest)
      {
         while ( start != stop )
         {
           *dest = *start;
           ++start;
           ++dest;
         }
      }
      copy kommer alltså alltid anta att intervallet som börjar med
      dest alltid är minst lika stort som antalet element i [start, stop).
      */

    SECTION("preallocated vector")
    {
        // copy kommer inte modifera v2s storlek. Vi måste se till att
        // v2 är nog stor
        vector<int> v2;
        v2.resize(v.size());
        copy(begin(v), end(v), begin(v2));
        CHECK(same_content(v,v2));
    }
     SECTION("back_insert_iterator")
    {
        // alternativt använder vi en speciell iterator (back_insert_iterator)
        // som anropar push_back när vi försöker tilldela den ett värde
        vector<int> v2;
        copy(begin(v), end(v), back_inserter(v2));
        CHECK(same_content(v,v2));
    }
    SECTION("iterator")
    {
        // initiering med iterator. Fördelen med denna variant är att
        // det inte måste vara samma typ. Kopieringskonstruktorn kräver
        // att du kopierar från ett objekt av samma typ, denna
        // konstruktor godkänner vilken Input-iterator som helst så
        // länge typen på de värden de refererar till är korrekt.
        vector<int> v2 { begin(v), end(v) };
        CHECK(same_content(v,v2));
    } 

}
