// Sätt in ett värde sorterat i en vector
#include <algorithm>
#include <iostream>
#include <vector>
#include <iterator>
using namespace std;

size_t find_position(vector<int> const & v, int val)
{
    for ( size_t pos{}; pos < v.size(); ++pos )
    {
        if ( v[pos] >= val )
            return pos;
    }
    return v.size();
}

int main()
{


    vector<int> values;
    for ( int i; cin >> i; )
    {
        /* tre sätt att hitta rätt position och stoppa in */
        // Enkel loop i en funktion
        {
            auto pos = find_position(values, i);
            // insert kräver iterator... använd att vi har RandomAccess
            auto it = begin(values) + pos;
            values.insert(it, i);
        }
        // find_if. Använder linjärsökning (dvs stegar ett element i 
        // taget tills den hittar rätt position). lambda-uttrycket
        // tar endast emot ett värde i taget från find_if (det 
        // algorimen undersöker för tillfället) och vi måste därför
        // binda "i" i capture-delen (hakparenteserna).
        {
            auto pos = find_if(begin(values), end(values),
                    [i](int val) {
                    return val >= i;
                    });
            values.insert(pos, i);
        }
        // lower_bound används på en sorterad mängd för att hitta
        // första värdet som är större eller lika med det sökta. 
        // Använder binärsökning (halverar sökmängden varje iteration)
        // detta är både effektivare, mer läsbart och säkrare!
        {
            auto pos { lower_bound(begin(values), end(values), i) };
            values.insert(pos, i);
        }
    }
    if ( is_sorted(begin(values), end(values) ) )
    {
        cout << "Sorterad" << endl;
    }
}
