#pragma once
#include <memory>
#include <stdexcept>

struct no_value_t {} const no_value{};

template <typename T>
class Optional
{
public:
    Optional() = default;
    Optional(Optional const & o);
    Optional & operator=(Optional const & o) &;
    Optional(Optional &&) = default;

    // initialize from underlying type
    Optional(T i);
    Optional & operator=(T i) &;

    // Set to no_value
    Optional(no_value_t) noexcept;
    Optional & operator=(no_value_t) noexcept;

    // accessors. value throws when unset, dereferencing unset value is undefined
    T & value();
    T const & value() const;
    T & operator*() noexcept;
    T const & operator*() const noexcept;

    // checker
    explicit operator bool() const;
    bool has_value() const;

    // utility functions
    void swap(Optional & o);
    void reset();
private:
    std::unique_ptr<T> data {};
};

class no_value_error: public std::runtime_error
{
    using std::runtime_error::runtime_error;
};

#include "optional.tcc"
