#include "optional.h"

#include <utility>

template<typename T>
Optional<T>::Optional(T i) : data { std::make_unique<T>( i )} {}

template<typename T>
Optional<T>::Optional(Optional<T> const& o)
{
    if (o.has_value())
    {
        data = std::make_unique<T>( *o );
    }
}

template<typename T>
Optional<T>& Optional<T>::operator=(Optional const& o)&
{
    Optional{ o }.swap(*this);
    return *this;
}

template<typename T>
Optional<T>& Optional<T>::operator=(T i)&
{
    if (!has_value())
    {
        data = std::make_unique<T>();
    }
    *data = i;
    return *this;
}

template<typename T>
Optional<T>::Optional(no_value_t) noexcept: data {} {}

template<typename T>
Optional<T>& Optional<T>::operator=(no_value_t) noexcept
{
    reset();
    return *this;
}

template<typename T>
T& Optional<T>::value()
{
    if (!has_value())
    {
        throw no_value_error{ "No value!" };
    }
    return *data;
}

template<typename T>
T const& Optional<T>::value() const
{
    if (!has_value())
    {
        throw no_value_error{ "No value!" };
    }
    return *data;
}

template<typename T>
Optional<T>::operator bool() const
{
    return has_value();
}

template<typename T>
bool Optional<T>::has_value() const
{
    return data != nullptr;
}

template<typename T>
T const& Optional<T>::operator*() const noexcept
{
    return *data;
}

template<typename T>
T& Optional<T>::operator*() noexcept
{
    return *data;
}

template<typename T>
void Optional<T>::swap(Optional& o)
{
    using std::swap;
    swap(data, o.data);
}

template<typename T>
void Optional<T>::reset()
{
    data.reset();
}

