#include "optional.h"
#include "catch.hpp"


TEST_CASE( "empty" )
{
    Optional<int> o;
    REQUIRE_FALSE(o.has_value());
    REQUIRE_THROWS_AS(o.value(), no_value_error);
    SECTION ("reassign")
    {
        o = 5;
        CHECK(o.value() == 5);
    }
}

TEST_CASE( "has value" )
{
    Optional<int> o{2};
    CHECK(o.has_value());
    CHECK(*o == 2);
    SECTION ("reassign")
    {
        o = 5;
        CHECK(o.value() == 5);
    }
    SECTION("no_value")
    {
        o = no_value;
        CHECK_FALSE(o.has_value());
    }
}

Optional<int> get_value(bool should_return)
{
    if (should_return)
    {
        return 3;
    }
    return no_value;
}

TEST_CASE("create in function")
{
    CHECK(get_value(true).has_value());
    CHECK_FALSE(get_value(false).has_value());

}
TEST_CASE( "string" )
{
    Optional<std::string> o{"hello"};
    CHECK(o.has_value());
    CHECK(*o == "hello");
    SECTION ("reassign")
    {
        o = "nu";
        CHECK(o.value() == "nu");
    }
    SECTION("no_value")
    {
        o = no_value;
        CHECK_FALSE(o.has_value());
    }
}

