#pragma once
#include <stdexcept>
#include <memory>

struct no_value_t {} inline const no_value{};

class Optional
{
public:
    Optional() = default;
    Optional(Optional const & o);
    Optional & operator=(Optional const & o) &;

    // initialize from underlying type
    Optional(int i);
    Optional & operator=(int i) &;

    // Set to no_value
    Optional(no_value_t) noexcept;
    Optional & operator=(no_value_t) noexcept;

    // accessors. value throws when unset, dereferencing unset value is undefined
    int & value();
    int const & value() const;
    int & operator*() noexcept;
    int const & operator*() const noexcept;

    // checker
    explicit operator bool() const;
    bool has_value() const;

    // utility functions
    void swap(Optional & o);
    void reset();
private:
    std::unique_ptr<int> data {};
};

class no_value_error: public std::runtime_error
{
    using std::runtime_error::runtime_error;
};
